@@ -16,21 +16,20 @@
 #define GEXIV2_STREAM_IO_H
 
 #include <gexiv2/gexiv2-managed-stream.h>
-#include <exiv2/basicio.hpp>
+#include <exiv2/exiv2.hpp>
 #include <gio/gio.h>
 
 
 class StreamIo : public Exiv2::BasicIo {
 public:
+#if EXIV2_TEST_VERSION(0,27,99)
+    using ptr_type = Exiv2::BasicIo::UniquePtr;
+#else
+    using ptr_type = Exiv2::BasicIo::AutoPtr;
+#endif
 
 	StreamIo (ManagedStreamCallbacks* cb);
 
-#if EXIV2_MAJOR_VERSION >= 1 || (EXIV2_MAJOR_VERSION == 0 && EXIV2_MINOR_VERSION >= 26)
-    typedef size_t size_type;
-#else
-    typedef long size_type;
-#endif
-
 	virtual ~StreamIo ();
 	virtual int open ();
 	virtual int close ();
@@ -45,7 +44,7 @@ public:
 	virtual Exiv2::byte* mmap (bool isWriteable = false);
 	virtual int munmap ();
 	virtual long tell () const;
-	virtual size_type size () const;
+	virtual size_t size () const;
 	virtual bool isopen () const;
 	virtual int error () const;
 	virtual bool eof () const;
@@ -53,15 +52,14 @@ public:
 #ifdef EXV_UNICODE_PATH
 	virtual std::wstring wpath () const;
 #endif
-	virtual BasicIo::AutoPtr temporary () const;
+	virtual ptr_type temporary () const;
 
 private:
-
 	/* stream callbacks */
 	ManagedStreamCallbacks* cb;
 	
 	/* used for mmap and  munmap */
-	Exiv2::BasicIo::AutoPtr memio;
+	ptr_type memio;
 
 	/* closing does not mean closing the stream, because this would
 	   destroy it. So just keep track about current state and let stream
